# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["SettingUpsertAvalaraCredentialsParams"]


class SettingUpsertAvalaraCredentialsParams(TypedDict, total=False):
    avalara_environment: Required[Literal["PRODUCTION", "SANDBOX"]]
    """The Avalara environment to use (SANDBOX or PRODUCTION)."""

    avalara_password: Required[str]
    """The password for the Avalara account."""

    avalara_username: Required[str]
    """The username for the Avalara account."""

    delivery_method_ids: Required[SequenceNotStr[str]]
    """
    The delivery method IDs of the billing provider configurations to update, can be
    found in the response of the `/listConfiguredBillingProviders` endpoint.
    """

    commit_transactions: bool
    """
    Commit transactions if you want Metronome tax calculations used for reporting
    and tax filings.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime

from ..._models import BaseModel
from .credit_ledger_entry import CreditLedgerEntry
from ..shared.credit_type_data import CreditTypeData

__all__ = ["CreditGrantListResponse", "Balance", "GrantAmount", "PaidAmount", "Product"]


class Balance(BaseModel):
    """
    The effective balance of the grant as of the end of the customer's current billing period. Expiration deductions will be included only if the grant expires before the end of the current billing period.
    """

    effective_at: datetime
    """The end_date of the customer's current billing period."""

    excluding_pending: float
    """The grant's current balance including all posted deductions.

    If the grant has expired, this amount will be 0.
    """

    including_pending: float
    """The grant's current balance including all posted and pending deductions.

    If the grant expires before the end of the customer's current billing period,
    this amount will be 0.
    """


class GrantAmount(BaseModel):
    """the amount of credits initially granted"""

    amount: float

    credit_type: CreditTypeData
    """the credit type for the amount granted"""


class PaidAmount(BaseModel):
    """the amount paid for this credit grant"""

    amount: float

    credit_type: CreditTypeData
    """the credit type for the amount paid"""


class Product(BaseModel):
    id: str

    name: str


class CreditGrantListResponse(BaseModel):
    id: str
    """the Metronome ID of the credit grant"""

    balance: Balance
    """
    The effective balance of the grant as of the end of the customer's current
    billing period. Expiration deductions will be included only if the grant expires
    before the end of the current billing period.
    """

    custom_fields: Dict[str, str]
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    customer_id: str
    """the Metronome ID of the customer"""

    deductions: List[CreditLedgerEntry]

    effective_at: datetime

    expires_at: datetime

    grant_amount: GrantAmount
    """the amount of credits initially granted"""

    name: str

    paid_amount: PaidAmount
    """the amount paid for this credit grant"""

    pending_deductions: List[CreditLedgerEntry]

    priority: float

    credit_grant_type: Optional[str] = None

    invoice_id: Optional[str] = None
    """
    the Metronome ID of the invoice with the purchase charge for this credit grant,
    if applicable
    """

    products: Optional[List[Product]] = None
    """The products which these credits will be applied to.

    (If unspecified, the credits will be applied to charges for all products.)
    """

    reason: Optional[str] = None

    uniqueness_key: Optional[str] = None
    """Prevents the creation of duplicates.

    If a request to create a record is made with a previously used uniqueness key, a
    new record will not be created and the request will fail with a 409 error.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo
from ..shared_params.tier import Tier
from ..shared_params.base_usage_filter import BaseUsageFilter
from ..shared_params.commit_specifier_input import CommitSpecifierInput
from ..shared_params.spend_threshold_configuration import SpendThresholdConfiguration
from ..shared_params.commit_hierarchy_configuration import CommitHierarchyConfiguration
from ..shared_params.prepaid_balance_threshold_configuration import PrepaidBalanceThresholdConfiguration

__all__ = [
    "ContractCreateParams",
    "BillingProviderConfiguration",
    "Commit",
    "CommitAccessSchedule",
    "CommitAccessScheduleScheduleItem",
    "CommitInvoiceSchedule",
    "CommitInvoiceScheduleRecurringSchedule",
    "CommitInvoiceScheduleScheduleItem",
    "CommitPaymentGateConfig",
    "CommitPaymentGateConfigPrecalculatedTaxConfig",
    "CommitPaymentGateConfigStripeConfig",
    "Credit",
    "CreditAccessSchedule",
    "CreditAccessScheduleScheduleItem",
    "Discount",
    "DiscountSchedule",
    "DiscountScheduleRecurringSchedule",
    "DiscountScheduleScheduleItem",
    "HierarchyConfiguration",
    "HierarchyConfigurationParent",
    "HierarchyConfigurationParentBehavior",
    "Override",
    "OverrideOverrideSpecifier",
    "OverrideOverwriteRate",
    "OverrideTier",
    "ProfessionalService",
    "RecurringCommit",
    "RecurringCommitAccessAmount",
    "RecurringCommitCommitDuration",
    "RecurringCommitInvoiceAmount",
    "RecurringCommitSubscriptionConfig",
    "RecurringCommitSubscriptionConfigApplySeatIncreaseConfig",
    "RecurringCredit",
    "RecurringCreditAccessAmount",
    "RecurringCreditCommitDuration",
    "RecurringCreditSubscriptionConfig",
    "RecurringCreditSubscriptionConfigApplySeatIncreaseConfig",
    "ResellerRoyalty",
    "ResellerRoyaltyAwsOptions",
    "ResellerRoyaltyGcpOptions",
    "RevenueSystemConfiguration",
    "ScheduledCharge",
    "ScheduledChargeSchedule",
    "ScheduledChargeScheduleRecurringSchedule",
    "ScheduledChargeScheduleScheduleItem",
    "Subscription",
    "SubscriptionProration",
    "SubscriptionSubscriptionRate",
    "SubscriptionSeatConfig",
    "Transition",
    "TransitionFutureInvoiceBehavior",
    "UsageStatementSchedule",
]


class ContractCreateParams(TypedDict, total=False):
    customer_id: Required[str]

    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """inclusive contract start time"""

    billing_provider_configuration: BillingProviderConfiguration
    """The billing provider configuration associated with a contract.

    Provide either an ID or the provider and delivery method.
    """

    commits: Iterable[Commit]

    credits: Iterable[Credit]

    custom_fields: Dict[str, str]
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    discounts: Iterable[Discount]
    """This field's availability is dependent on your client's configuration."""

    ending_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """exclusive contract end time"""

    hierarchy_configuration: HierarchyConfiguration

    multiplier_override_prioritization: Literal["LOWEST_MULTIPLIER", "EXPLICIT"]
    """
    Defaults to LOWEST_MULTIPLIER, which applies the greatest discount to list
    prices automatically. EXPLICIT prioritization requires specifying priorities for
    each multiplier; the one with the lowest priority value will be prioritized
    first. If tiered overrides are used, prioritization must be explicit.
    """

    name: str

    net_payment_terms_days: float

    netsuite_sales_order_id: str
    """This field's availability is dependent on your client's configuration."""

    overrides: Iterable[Override]

    prepaid_balance_threshold_configuration: PrepaidBalanceThresholdConfiguration

    priority: float
    """Priority of the contract."""

    professional_services: Iterable[ProfessionalService]
    """This field's availability is dependent on your client's configuration."""

    rate_card_alias: str
    """
    Selects the rate card linked to the specified alias as of the contract's start
    date.
    """

    rate_card_id: str

    recurring_commits: Iterable[RecurringCommit]

    recurring_credits: Iterable[RecurringCredit]

    reseller_royalties: Iterable[ResellerRoyalty]
    """This field's availability is dependent on your client's configuration."""

    revenue_system_configuration: RevenueSystemConfiguration
    """The revenue system configuration associated with a contract.

    Provide either an ID or the provider and delivery method.
    """

    salesforce_opportunity_id: str
    """This field's availability is dependent on your client's configuration."""

    scheduled_charges: Iterable[ScheduledCharge]

    scheduled_charges_on_usage_invoices: Literal["ALL"]
    """
    Determines which scheduled and commit charges to consolidate onto the Contract's
    usage invoice. The charge's `timestamp` must match the usage invoice's
    `ending_before` date for consolidation to occur. This field cannot be modified
    after a Contract has been created. If this field is omitted, charges will appear
    on a separate invoice from usage charges.
    """

    spend_threshold_configuration: SpendThresholdConfiguration

    subscriptions: Iterable[Subscription]
    """
    Optional list of
    [subscriptions](https://docs.metronome.com/manage-product-access/create-subscription/)
    to add to the contract.
    """

    total_contract_value: float
    """This field's availability is dependent on your client's configuration."""

    transition: Transition

    uniqueness_key: str
    """Prevents the creation of duplicates.

    If a request to create a record is made with a previously used uniqueness key, a
    new record will not be created and the request will fail with a 409 error.
    """

    usage_filter: BaseUsageFilter

    usage_statement_schedule: UsageStatementSchedule


class BillingProviderConfiguration(TypedDict, total=False):
    """The billing provider configuration associated with a contract.

    Provide either an ID or the provider and delivery method.
    """

    billing_provider: Literal["aws_marketplace", "azure_marketplace", "gcp_marketplace", "stripe", "netsuite"]
    """Do not specify if using billing_provider_configuration_id."""

    billing_provider_configuration_id: str
    """The Metronome ID of the billing provider configuration.

    Use when a customer has multiple configurations with the same billing provider
    and delivery method. Otherwise, specify the billing_provider and
    delivery_method.
    """

    delivery_method: Literal["direct_to_billing_provider", "aws_sqs", "tackle", "aws_sns"]
    """Do not specify if using billing_provider_configuration_id."""


class CommitAccessScheduleScheduleItem(TypedDict, total=False):
    amount: Required[float]

    ending_before: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp (exclusive)"""

    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp (inclusive)"""


class CommitAccessSchedule(TypedDict, total=False):
    """Required: Schedule for distributing the commit to the customer.

    For "POSTPAID" commits only one schedule item is allowed and amount must match invoice_schedule total.
    """

    schedule_items: Required[Iterable[CommitAccessScheduleScheduleItem]]

    credit_type_id: str
    """Defaults to USD (cents) if not passed"""


class CommitInvoiceScheduleRecurringSchedule(TypedDict, total=False):
    """Enter the unit price and quantity for the charge or instead only send the amount.

    If amount is sent, the unit price is assumed to be the amount and quantity is inferred to be 1.
    """

    amount_distribution: Required[Literal["DIVIDED", "DIVIDED_ROUNDED", "EACH"]]

    ending_before: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp (exclusive)."""

    frequency: Required[Literal["MONTHLY", "QUARTERLY", "SEMI_ANNUAL", "ANNUAL"]]

    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp (inclusive)."""

    amount: float
    """Amount for the charge.

    Can be provided instead of unit_price and quantity. If amount is sent, the
    unit_price is assumed to be the amount and quantity is inferred to be 1.
    """

    quantity: float
    """Quantity for the charge.

    Will be multiplied by unit_price to determine the amount and must be specified
    with unit_price. If specified amount cannot be provided.
    """

    unit_price: float
    """Unit price for the charge.

    Will be multiplied by quantity to determine the amount and must be specified
    with quantity. If specified amount cannot be provided.
    """


class CommitInvoiceScheduleScheduleItem(TypedDict, total=False):
    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """timestamp of the scheduled event"""

    amount: float
    """Amount for the charge.

    Can be provided instead of unit_price and quantity. If amount is sent, the
    unit_price is assumed to be the amount and quantity is inferred to be 1.
    """

    quantity: float
    """Quantity for the charge.

    Will be multiplied by unit_price to determine the amount and must be specified
    with unit_price. If specified amount cannot be provided.
    """

    unit_price: float
    """Unit price for the charge.

    Will be multiplied by quantity to determine the amount and must be specified
    with quantity. If specified amount cannot be provided.
    """


class CommitInvoiceSchedule(TypedDict, total=False):
    """
    Required for "POSTPAID" commits: the true up invoice will be generated at this time and only one schedule item is allowed; the total must match access_schedule amount. Optional for "PREPAID" commits: if not provided, this will be a "complimentary" commit with no invoice.
    """

    credit_type_id: str
    """Defaults to USD (cents) if not passed."""

    do_not_invoice: bool
    """This field is only applicable to commit invoice schedules.

    If true, this schedule will not generate an invoice.
    """

    recurring_schedule: CommitInvoiceScheduleRecurringSchedule
    """Enter the unit price and quantity for the charge or instead only send the
    amount.

    If amount is sent, the unit price is assumed to be the amount and quantity is
    inferred to be 1.
    """

    schedule_items: Iterable[CommitInvoiceScheduleScheduleItem]
    """Either provide amount or provide both unit_price and quantity."""


class CommitPaymentGateConfigPrecalculatedTaxConfig(TypedDict, total=False):
    """Only applicable if using PRECALCULATED as your tax type."""

    tax_amount: Required[float]
    """Amount of tax to be applied.

    This should be in the same currency and denomination as the commit's invoice
    schedule
    """

    tax_name: str
    """Name of the tax to be applied.

    This may be used in an invoice line item description.
    """


class CommitPaymentGateConfigStripeConfig(TypedDict, total=False):
    """Only applicable if using STRIPE as your payment gate type."""

    payment_type: Required[Literal["INVOICE", "PAYMENT_INTENT"]]
    """If left blank, will default to INVOICE"""

    invoice_metadata: Dict[str, str]
    """Metadata to be added to the Stripe invoice.

    Only applicable if using INVOICE as your payment type.
    """

    on_session_payment: bool
    """If true, the payment will be made assuming the customer is present (i.e.

    on session).

    If false, the payment will be made assuming the customer is not present (i.e.
    off session). For cardholders from a country with an e-mandate requirement (e.g.
    India), the payment may be declined.

    If left blank, will default to false.
    """


class CommitPaymentGateConfig(TypedDict, total=False):
    """optionally payment gate this commit"""

    payment_gate_type: Required[Literal["NONE", "STRIPE", "EXTERNAL"]]
    """Gate access to the commit balance based on successful collection of payment.

    Select STRIPE for Metronome to facilitate payment via Stripe. Select EXTERNAL to
    facilitate payment using your own payment integration. Select NONE if you do not
    wish to payment gate the commit balance.
    """

    precalculated_tax_config: CommitPaymentGateConfigPrecalculatedTaxConfig
    """Only applicable if using PRECALCULATED as your tax type."""

    stripe_config: CommitPaymentGateConfigStripeConfig
    """Only applicable if using STRIPE as your payment gate type."""

    tax_type: Literal["NONE", "STRIPE", "ANROK", "AVALARA", "PRECALCULATED"]
    """Stripe tax is only supported for Stripe payment gateway.

    Select NONE if you do not wish Metronome to calculate tax on your behalf.
    Leaving this field blank will default to NONE.
    """


class Commit(TypedDict, total=False):
    product_id: Required[str]

    type: Required[Literal["PREPAID", "POSTPAID"]]

    access_schedule: CommitAccessSchedule
    """Required: Schedule for distributing the commit to the customer.

    For "POSTPAID" commits only one schedule item is allowed and amount must match
    invoice_schedule total.
    """

    amount: float
    """(DEPRECATED) Use access_schedule and invoice_schedule instead."""

    applicable_product_ids: SequenceNotStr[str]
    """Which products the commit applies to.

    If applicable_product_ids, applicable_product_tags or specifiers are not
    provided, the commit applies to all products.
    """

    applicable_product_tags: SequenceNotStr[str]
    """Which tags the commit applies to.

    If applicable_product_ids, applicable_product_tags or specifiers are not
    provided, the commit applies to all products.
    """

    custom_fields: Dict[str, str]
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    description: str
    """Used only in UI/API. It is not exposed to end customers."""

    hierarchy_configuration: CommitHierarchyConfiguration
    """Optional configuration for commit hierarchy access control"""

    invoice_schedule: CommitInvoiceSchedule
    """
    Required for "POSTPAID" commits: the true up invoice will be generated at this
    time and only one schedule item is allowed; the total must match access_schedule
    amount. Optional for "PREPAID" commits: if not provided, this will be a
    "complimentary" commit with no invoice.
    """

    name: str
    """displayed on invoices"""

    netsuite_sales_order_id: str
    """This field's availability is dependent on your client's configuration."""

    payment_gate_config: CommitPaymentGateConfig
    """optionally payment gate this commit"""

    priority: float
    """
    If multiple commits are applicable, the one with the lower priority will apply
    first.
    """

    rate_type: Literal["COMMIT_RATE", "LIST_RATE"]

    rollover_fraction: float
    """Fraction of unused segments that will be rolled over. Must be between 0 and 1."""

    specifiers: Iterable[CommitSpecifierInput]
    """
    List of filters that determine what kind of customer usage draws down a commit
    or credit. A customer's usage needs to meet the condition of at least one of the
    specifiers to contribute to a commit's or credit's drawdown. This field cannot
    be used together with `applicable_product_ids` or `applicable_product_tags`.
    """

    temporary_id: str
    """
    A temporary ID for the commit that can be used to reference the commit for
    commit specific overrides.
    """


class CreditAccessScheduleScheduleItem(TypedDict, total=False):
    amount: Required[float]

    ending_before: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp (exclusive)"""

    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp (inclusive)"""


class CreditAccessSchedule(TypedDict, total=False):
    """Schedule for distributing the credit to the customer."""

    schedule_items: Required[Iterable[CreditAccessScheduleScheduleItem]]

    credit_type_id: str
    """Defaults to USD (cents) if not passed"""


class Credit(TypedDict, total=False):
    access_schedule: Required[CreditAccessSchedule]
    """Schedule for distributing the credit to the customer."""

    product_id: Required[str]

    applicable_product_ids: SequenceNotStr[str]
    """Which products the credit applies to.

    If both applicable_product_ids and applicable_product_tags are not provided, the
    credit applies to all products.
    """

    applicable_product_tags: SequenceNotStr[str]
    """Which tags the credit applies to.

    If both applicable_product_ids and applicable_product_tags are not provided, the
    credit applies to all products.
    """

    custom_fields: Dict[str, str]
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    description: str
    """Used only in UI/API. It is not exposed to end customers."""

    hierarchy_configuration: CommitHierarchyConfiguration
    """Optional configuration for credit hierarchy access control"""

    name: str
    """displayed on invoices"""

    netsuite_sales_order_id: str
    """This field's availability is dependent on your client's configuration."""

    priority: float
    """
    If multiple credits are applicable, the one with the lower priority will apply
    first.
    """

    rate_type: Literal["COMMIT_RATE", "LIST_RATE"]

    specifiers: Iterable[CommitSpecifierInput]
    """
    List of filters that determine what kind of customer usage draws down a commit
    or credit. A customer's usage needs to meet the condition of at least one of the
    specifiers to contribute to a commit's or credit's drawdown. This field cannot
    be used together with `applicable_product_ids` or `applicable_product_tags`.
    """


class DiscountScheduleRecurringSchedule(TypedDict, total=False):
    """Enter the unit price and quantity for the charge or instead only send the amount.

    If amount is sent, the unit price is assumed to be the amount and quantity is inferred to be 1.
    """

    amount_distribution: Required[Literal["DIVIDED", "DIVIDED_ROUNDED", "EACH"]]

    ending_before: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp (exclusive)."""

    frequency: Required[Literal["MONTHLY", "QUARTERLY", "SEMI_ANNUAL", "ANNUAL"]]

    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp (inclusive)."""

    amount: float
    """Amount for the charge.

    Can be provided instead of unit_price and quantity. If amount is sent, the
    unit_price is assumed to be the amount and quantity is inferred to be 1.
    """

    quantity: float
    """Quantity for the charge.

    Will be multiplied by unit_price to determine the amount and must be specified
    with unit_price. If specified amount cannot be provided.
    """

    unit_price: float
    """Unit price for the charge.

    Will be multiplied by quantity to determine the amount and must be specified
    with quantity. If specified amount cannot be provided.
    """


class DiscountScheduleScheduleItem(TypedDict, total=False):
    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """timestamp of the scheduled event"""

    amount: float
    """Amount for the charge.

    Can be provided instead of unit_price and quantity. If amount is sent, the
    unit_price is assumed to be the amount and quantity is inferred to be 1.
    """

    quantity: float
    """Quantity for the charge.

    Will be multiplied by unit_price to determine the amount and must be specified
    with unit_price. If specified amount cannot be provided.
    """

    unit_price: float
    """Unit price for the charge.

    Will be multiplied by quantity to determine the amount and must be specified
    with quantity. If specified amount cannot be provided.
    """


class DiscountSchedule(TypedDict, total=False):
    """Must provide either schedule_items or recurring_schedule."""

    credit_type_id: str
    """Defaults to USD (cents) if not passed."""

    do_not_invoice: bool
    """This field is only applicable to commit invoice schedules.

    If true, this schedule will not generate an invoice.
    """

    recurring_schedule: DiscountScheduleRecurringSchedule
    """Enter the unit price and quantity for the charge or instead only send the
    amount.

    If amount is sent, the unit price is assumed to be the amount and quantity is
    inferred to be 1.
    """

    schedule_items: Iterable[DiscountScheduleScheduleItem]
    """Either provide amount or provide both unit_price and quantity."""


class Discount(TypedDict, total=False):
    product_id: Required[str]

    schedule: Required[DiscountSchedule]
    """Must provide either schedule_items or recurring_schedule."""

    custom_fields: Dict[str, str]
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    name: str
    """displayed on invoices"""

    netsuite_sales_order_id: str
    """This field's availability is dependent on your client's configuration."""


class HierarchyConfigurationParent(TypedDict, total=False):
    contract_id: Required[str]

    customer_id: Required[str]


class HierarchyConfigurationParentBehavior(TypedDict, total=False):
    invoice_consolidation_type: Literal["CONCATENATE", "NONE"]
    """
    Account hierarchy M3 - Indicates the desired behavior of consolidated invoices
    generated by the parent in a customer hierarchy

    **CONCATENATE**: Statements on the invoices of child customers will be appended
    to the consolidated invoice

    **NONE**: Do not generate consolidated invoices
    """


class HierarchyConfiguration(TypedDict, total=False):
    parent: HierarchyConfigurationParent

    parent_behavior: HierarchyConfigurationParentBehavior

    payer: Literal["SELF", "PARENT"]
    """
    Account hierarchy M3 - Indicates which customer should pay for the child's
    invoice charges

    **SELF**: The child pays for its own invoice charges

    **PARENT**: The parent pays for the child's invoice charges
    """

    usage_statement_behavior: Literal["CONSOLIDATE", "SEPARATE"]
    """
    Account hierarchy M3 - Indicates the behavior of the child's invoice statements
    on the parent's invoices.

    **CONSOLIDATE**: Child's invoice statements will be added to parent's
    consolidated invoices

    **SEPARATE**: Child's invoice statements will appear not appear on parent's
    consolidated invoices
    """


class OverrideOverrideSpecifier(TypedDict, total=False):
    billing_frequency: Literal["MONTHLY", "QUARTERLY", "ANNUAL", "WEEKLY"]

    commit_ids: SequenceNotStr[str]
    """Can only be used for commit specific overrides.

    Must be used in conjunction with one of `product_id`, `product_tags`,
    `pricing_group_values`, or `presentation_group_values`. If provided, the
    override will only apply to the specified commits. If not provided, the override
    will apply to all commits.
    """

    presentation_group_values: Dict[str, str]
    """A map of group names to values.

    The override will only apply to line items with the specified presentation group
    values.
    """

    pricing_group_values: Dict[str, str]
    """A map of pricing group names to values.

    The override will only apply to products with the specified pricing group
    values.
    """

    product_id: str
    """If provided, the override will only apply to the product with the specified ID."""

    product_tags: SequenceNotStr[str]
    """
    If provided, the override will only apply to products with all the specified
    tags.
    """

    recurring_commit_ids: SequenceNotStr[str]
    """Can only be used for commit specific overrides.

    Must be used in conjunction with one of `product_id`, `product_tags`,
    `pricing_group_values`, or `presentation_group_values`. If provided, the
    override will only apply to commits created by the specified recurring commit
    ids.
    """

    recurring_credit_ids: SequenceNotStr[str]
    """Can only be used for commit specific overrides.

    Must be used in conjunction with one of `product_id`, `product_tags`,
    `pricing_group_values`, or `presentation_group_values`. If provided, the
    override will only apply to credits created by the specified recurring credit
    ids.
    """


class OverrideOverwriteRate(TypedDict, total=False):
    """Required for OVERWRITE type."""

    rate_type: Required[Literal["FLAT", "PERCENTAGE", "SUBSCRIPTION", "TIERED", "CUSTOM"]]

    credit_type_id: str

    custom_rate: Dict[str, object]
    """Only set for CUSTOM rate_type.

    This field is interpreted by custom rate processors.
    """

    is_prorated: bool
    """Default proration configuration.

    Only valid for SUBSCRIPTION rate_type. Must be set to true.
    """

    price: float
    """Default price.

    For FLAT rate_type, this must be >=0. For PERCENTAGE rate_type, this is a
    decimal fraction, e.g. use 0.1 for 10%; this must be >=0 and <=1.
    """

    quantity: float
    """Default quantity. For SUBSCRIPTION rate_type, this must be >=0."""

    tiers: Iterable[Tier]
    """Only set for TIERED rate_type."""


class OverrideTier(TypedDict, total=False):
    multiplier: Required[float]

    size: float


class Override(TypedDict, total=False):
    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp indicating when the override will start applying (inclusive)"""

    applicable_product_tags: SequenceNotStr[str]
    """tags identifying products whose rates are being overridden.

    Cannot be used in conjunction with override_specifiers.
    """

    ending_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """RFC 3339 timestamp indicating when the override will stop applying (exclusive)"""

    entitled: bool

    is_commit_specific: bool
    """Indicates whether the override should only apply to commits.

    Defaults to `false`. If `true`, you can specify relevant commits in
    `override_specifiers` by passing `commit_ids`. if you do not specify
    `commit_ids`, then the override will apply when consuming any prepaid or
    postpaid commit.
    """

    multiplier: float
    """Required for MULTIPLIER type. Must be >=0."""

    override_specifiers: Iterable[OverrideOverrideSpecifier]
    """Cannot be used in conjunction with product_id or applicable_product_tags.

    If provided, the override will apply to all products with the specified
    specifiers.
    """

    overwrite_rate: OverrideOverwriteRate
    """Required for OVERWRITE type."""

    priority: float
    """Required for EXPLICIT multiplier prioritization scheme and all TIERED overrides.

    Under EXPLICIT prioritization, overwrites are prioritized first, and then tiered
    and multiplier overrides are prioritized by their priority value (lowest first).
    Must be > 0.
    """

    product_id: str
    """ID of the product whose rate is being overridden.

    Cannot be used in conjunction with override_specifiers.
    """

    target: Literal["COMMIT_RATE", "LIST_RATE"]
    """Indicates whether the override applies to commit rates or list rates.

    Can only be used for overrides that have `is_commit_specific` set to `true`.
    Defaults to `"LIST_RATE"`.
    """

    tiers: Iterable[OverrideTier]
    """Required for TIERED type. Must have at least one tier."""

    type: Literal["OVERWRITE", "MULTIPLIER", "TIERED"]
    """Overwrites are prioritized over multipliers and tiered overrides."""


class ProfessionalService(TypedDict, total=False):
    max_amount: Required[float]
    """Maximum amount for the term."""

    product_id: Required[str]

    quantity: Required[float]
    """Quantity for the charge.

    Will be multiplied by unit_price to determine the amount.
    """

    unit_price: Required[float]
    """Unit price for the charge.

    Will be multiplied by quantity to determine the amount and must be specified.
    """

    custom_fields: Dict[str, str]
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    description: str

    netsuite_sales_order_id: str
    """This field's availability is dependent on your client's configuration."""


class RecurringCommitAccessAmount(TypedDict, total=False):
    """The amount of commit to grant."""

    credit_type_id: Required[str]

    unit_price: Required[float]

    quantity: float
    """
    This field is required unless a subscription is attached via
    `subscription_config`.
    """


class RecurringCommitCommitDuration(TypedDict, total=False):
    """Defines the length of the access schedule for each created commit/credit.

    The value represents the number of units. Unit defaults to "PERIODS", where the length of a period is determined by the recurrence_frequency.
    """

    value: Required[float]

    unit: Literal["PERIODS"]


class RecurringCommitInvoiceAmount(TypedDict, total=False):
    """The amount the customer should be billed for the commit. Not required."""

    credit_type_id: Required[str]

    quantity: Required[float]

    unit_price: Required[float]


class RecurringCommitSubscriptionConfigApplySeatIncreaseConfig(TypedDict, total=False):
    is_prorated: Required[bool]
    """Indicates whether a mid-period seat increase should be prorated."""


class RecurringCommitSubscriptionConfig(TypedDict, total=False):
    """Attach a subscription to the recurring commit/credit."""

    apply_seat_increase_config: Required[RecurringCommitSubscriptionConfigApplySeatIncreaseConfig]

    subscription_id: Required[str]
    """ID of the subscription to configure on the recurring commit/credit."""

    allocation: Literal["INDIVIDUAL", "POOLED"]
    """If set to POOLED, allocation added per seat is pooled across the account.

    If set to INDIVIDUAL, each seat in the subscription will have its own
    allocation.
    """


class RecurringCommit(TypedDict, total=False):
    access_amount: Required[RecurringCommitAccessAmount]
    """The amount of commit to grant."""

    commit_duration: Required[RecurringCommitCommitDuration]
    """Defines the length of the access schedule for each created commit/credit.

    The value represents the number of units. Unit defaults to "PERIODS", where the
    length of a period is determined by the recurrence_frequency.
    """

    priority: Required[float]
    """Will be passed down to the individual commits"""

    product_id: Required[str]

    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """determines the start time for the first commit"""

    applicable_product_ids: SequenceNotStr[str]
    """Will be passed down to the individual commits"""

    applicable_product_tags: SequenceNotStr[str]
    """Will be passed down to the individual commits"""

    description: str
    """Will be passed down to the individual commits"""

    ending_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Determines when the contract will stop creating recurring commits. optional"""

    hierarchy_configuration: CommitHierarchyConfiguration
    """Optional configuration for recurring commit/credit hierarchy access control"""

    invoice_amount: RecurringCommitInvoiceAmount
    """The amount the customer should be billed for the commit. Not required."""

    name: str
    """displayed on invoices. will be passed through to the individual commits"""

    netsuite_sales_order_id: str
    """Will be passed down to the individual commits"""

    proration: Literal["NONE", "FIRST", "LAST", "FIRST_AND_LAST"]
    """Determines whether the first and last commit will be prorated.

    If not provided, the default is FIRST_AND_LAST (i.e. prorate both the first and
    last commits).
    """

    rate_type: Literal["COMMIT_RATE", "LIST_RATE"]
    """Whether the created commits will use the commit rate or list rate"""

    recurrence_frequency: Literal["MONTHLY", "QUARTERLY", "ANNUAL", "WEEKLY"]
    """The frequency at which the recurring commits will be created.

    If not provided: - The commits will be created on the usage invoice frequency.
    If provided: - The period defined in the duration will correspond to this
    frequency. - Commits will be created aligned with the recurring commit's
    starting_at rather than the usage invoice dates.
    """

    rollover_fraction: float
    """Will be passed down to the individual commits.

    This controls how much of an individual unexpired commit will roll over upon
    contract transition. Must be between 0 and 1.
    """

    specifiers: Iterable[CommitSpecifierInput]
    """
    List of filters that determine what kind of customer usage draws down a commit
    or credit. A customer's usage needs to meet the condition of at least one of the
    specifiers to contribute to a commit's or credit's drawdown. This field cannot
    be used together with `applicable_product_ids` or `applicable_product_tags`.
    """

    subscription_config: RecurringCommitSubscriptionConfig
    """Attach a subscription to the recurring commit/credit."""

    temporary_id: str
    """
    A temporary ID that can be used to reference the recurring commit for commit
    specific overrides.
    """


class RecurringCreditAccessAmount(TypedDict, total=False):
    """The amount of commit to grant."""

    credit_type_id: Required[str]

    unit_price: Required[float]

    quantity: float
    """
    This field is required unless a subscription is attached via
    `subscription_config`.
    """


class RecurringCreditCommitDuration(TypedDict, total=False):
    """Defines the length of the access schedule for each created commit/credit.

    The value represents the number of units. Unit defaults to "PERIODS", where the length of a period is determined by the recurrence_frequency.
    """

    value: Required[float]

    unit: Literal["PERIODS"]


class RecurringCreditSubscriptionConfigApplySeatIncreaseConfig(TypedDict, total=False):
    is_prorated: Required[bool]
    """Indicates whether a mid-period seat increase should be prorated."""


class RecurringCreditSubscriptionConfig(TypedDict, total=False):
    """Attach a subscription to the recurring commit/credit."""

    apply_seat_increase_config: Required[RecurringCreditSubscriptionConfigApplySeatIncreaseConfig]

    subscription_id: Required[str]
    """ID of the subscription to configure on the recurring commit/credit."""

    allocation: Literal["INDIVIDUAL", "POOLED"]
    """If set to POOLED, allocation added per seat is pooled across the account.

    If set to INDIVIDUAL, each seat in the subscription will have its own
    allocation.
    """


class RecurringCredit(TypedDict, total=False):
    access_amount: Required[RecurringCreditAccessAmount]
    """The amount of commit to grant."""

    commit_duration: Required[RecurringCreditCommitDuration]
    """Defines the length of the access schedule for each created commit/credit.

    The value represents the number of units. Unit defaults to "PERIODS", where the
    length of a period is determined by the recurrence_frequency.
    """

    priority: Required[float]
    """Will be passed down to the individual commits"""

    product_id: Required[str]

    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """determines the start time for the first commit"""

    applicable_product_ids: SequenceNotStr[str]
    """Will be passed down to the individual commits"""

    applicable_product_tags: SequenceNotStr[str]
    """Will be passed down to the individual commits"""

    description: str
    """Will be passed down to the individual commits"""

    ending_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Determines when the contract will stop creating recurring commits. optional"""

    hierarchy_configuration: CommitHierarchyConfiguration
    """Optional configuration for recurring commit/credit hierarchy access control"""

    name: str
    """displayed on invoices. will be passed through to the individual commits"""

    netsuite_sales_order_id: str
    """Will be passed down to the individual commits"""

    proration: Literal["NONE", "FIRST", "LAST", "FIRST_AND_LAST"]
    """Determines whether the first and last commit will be prorated.

    If not provided, the default is FIRST_AND_LAST (i.e. prorate both the first and
    last commits).
    """

    rate_type: Literal["COMMIT_RATE", "LIST_RATE"]
    """Whether the created commits will use the commit rate or list rate"""

    recurrence_frequency: Literal["MONTHLY", "QUARTERLY", "ANNUAL", "WEEKLY"]
    """The frequency at which the recurring commits will be created.

    If not provided: - The commits will be created on the usage invoice frequency.
    If provided: - The period defined in the duration will correspond to this
    frequency. - Commits will be created aligned with the recurring commit's
    starting_at rather than the usage invoice dates.
    """

    rollover_fraction: float
    """Will be passed down to the individual commits.

    This controls how much of an individual unexpired commit will roll over upon
    contract transition. Must be between 0 and 1.
    """

    specifiers: Iterable[CommitSpecifierInput]
    """
    List of filters that determine what kind of customer usage draws down a commit
    or credit. A customer's usage needs to meet the condition of at least one of the
    specifiers to contribute to a commit's or credit's drawdown. This field cannot
    be used together with `applicable_product_ids` or `applicable_product_tags`.
    """

    subscription_config: RecurringCreditSubscriptionConfig
    """Attach a subscription to the recurring commit/credit."""

    temporary_id: str
    """
    A temporary ID that can be used to reference the recurring commit for commit
    specific overrides.
    """


class ResellerRoyaltyAwsOptions(TypedDict, total=False):
    aws_account_number: str

    aws_offer_id: str

    aws_payer_reference_id: str


class ResellerRoyaltyGcpOptions(TypedDict, total=False):
    gcp_account_id: str

    gcp_offer_id: str


class ResellerRoyalty(TypedDict, total=False):
    fraction: Required[float]

    netsuite_reseller_id: Required[str]

    reseller_type: Required[Literal["AWS", "AWS_PRO_SERVICE", "GCP", "GCP_PRO_SERVICE"]]

    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    applicable_product_ids: SequenceNotStr[str]
    """Must provide at least one of applicable_product_ids or applicable_product_tags."""

    applicable_product_tags: SequenceNotStr[str]
    """Must provide at least one of applicable_product_ids or applicable_product_tags."""

    aws_options: ResellerRoyaltyAwsOptions

    ending_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    gcp_options: ResellerRoyaltyGcpOptions

    reseller_contract_value: float


class RevenueSystemConfiguration(TypedDict, total=False):
    """The revenue system configuration associated with a contract.

    Provide either an ID or the provider and delivery method.
    """

    delivery_method: Literal["direct_to_billing_provider"]
    """How revenue recognition records should be delivered to the revenue system.

    Do not specify if using revenue_system_configuration_id.
    """

    provider: Literal["netsuite"]
    """The system that is providing services for revenue recognition.

    Do not specify if using revenue_system_configuration_id.
    """

    revenue_system_configuration_id: str
    """The Metronome ID of the revenue system configuration.

    Use when a customer has multiple configurations with the same provider and
    delivery method. Otherwise, specify the provider and delivery_method.
    """


class ScheduledChargeScheduleRecurringSchedule(TypedDict, total=False):
    """Enter the unit price and quantity for the charge or instead only send the amount.

    If amount is sent, the unit price is assumed to be the amount and quantity is inferred to be 1.
    """

    amount_distribution: Required[Literal["DIVIDED", "DIVIDED_ROUNDED", "EACH"]]

    ending_before: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp (exclusive)."""

    frequency: Required[Literal["MONTHLY", "QUARTERLY", "SEMI_ANNUAL", "ANNUAL"]]

    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp (inclusive)."""

    amount: float
    """Amount for the charge.

    Can be provided instead of unit_price and quantity. If amount is sent, the
    unit_price is assumed to be the amount and quantity is inferred to be 1.
    """

    quantity: float
    """Quantity for the charge.

    Will be multiplied by unit_price to determine the amount and must be specified
    with unit_price. If specified amount cannot be provided.
    """

    unit_price: float
    """Unit price for the charge.

    Will be multiplied by quantity to determine the amount and must be specified
    with quantity. If specified amount cannot be provided.
    """


class ScheduledChargeScheduleScheduleItem(TypedDict, total=False):
    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """timestamp of the scheduled event"""

    amount: float
    """Amount for the charge.

    Can be provided instead of unit_price and quantity. If amount is sent, the
    unit_price is assumed to be the amount and quantity is inferred to be 1.
    """

    quantity: float
    """Quantity for the charge.

    Will be multiplied by unit_price to determine the amount and must be specified
    with unit_price. If specified amount cannot be provided.
    """

    unit_price: float
    """Unit price for the charge.

    Will be multiplied by quantity to determine the amount and must be specified
    with quantity. If specified amount cannot be provided.
    """


class ScheduledChargeSchedule(TypedDict, total=False):
    """Must provide either schedule_items or recurring_schedule."""

    credit_type_id: str
    """Defaults to USD (cents) if not passed."""

    do_not_invoice: bool
    """This field is only applicable to commit invoice schedules.

    If true, this schedule will not generate an invoice.
    """

    recurring_schedule: ScheduledChargeScheduleRecurringSchedule
    """Enter the unit price and quantity for the charge or instead only send the
    amount.

    If amount is sent, the unit price is assumed to be the amount and quantity is
    inferred to be 1.
    """

    schedule_items: Iterable[ScheduledChargeScheduleScheduleItem]
    """Either provide amount or provide both unit_price and quantity."""


class ScheduledCharge(TypedDict, total=False):
    product_id: Required[str]

    schedule: Required[ScheduledChargeSchedule]
    """Must provide either schedule_items or recurring_schedule."""

    custom_fields: Dict[str, str]
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    name: str
    """displayed on invoices"""

    netsuite_sales_order_id: str
    """This field's availability is dependent on your client's configuration."""


class SubscriptionProration(TypedDict, total=False):
    invoice_behavior: Literal["BILL_IMMEDIATELY", "BILL_ON_NEXT_COLLECTION_DATE"]
    """Indicates how mid-period quantity adjustments are invoiced.

    **BILL_IMMEDIATELY**: Only available when collection schedule is `ADVANCE`. The
    quantity increase will be billed immediately on the scheduled date.
    **BILL_ON_NEXT_COLLECTION_DATE**: The quantity increase will be billed for
    in-arrears at the end of the period.
    """

    is_prorated: bool
    """Indicates if the partial period will be prorated or charged a full amount."""


class SubscriptionSubscriptionRate(TypedDict, total=False):
    billing_frequency: Required[Literal["MONTHLY", "QUARTERLY", "ANNUAL", "WEEKLY"]]
    """Frequency to bill subscription with.

    Together with product_id, must match existing rate on the rate card.
    """

    product_id: Required[str]
    """Must be subscription type product"""


class SubscriptionSeatConfig(TypedDict, total=False):
    initial_seat_ids: Required[SequenceNotStr[str]]
    """The initial assigned seats on this subscription."""

    seat_group_key: Required[str]
    """
    The property name, sent on usage events, that identifies the seat ID associated
    with the usage event. For example, the property name might be seat_id or
    user_id. The property must be set as a group key on billable metrics and a
    presentation/pricing group key on contract products. This allows linked
    recurring credits with an allocation per seat to be consumed by only one seat's
    usage.
    """

    initial_unassigned_seats: float
    """The initial amount of unassigned seats on this subscription."""


class Subscription(TypedDict, total=False):
    collection_schedule: Required[Literal["ADVANCE", "ARREARS"]]

    proration: Required[SubscriptionProration]

    subscription_rate: Required[SubscriptionSubscriptionRate]

    custom_fields: Dict[str, str]
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    description: str

    ending_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Exclusive end time for the subscription.

    If not provided, subscription inherits contract end date.
    """

    initial_quantity: float
    """The initial quantity for the subscription.

    It must be non-negative value. Required if quantity_management_mode is
    QUANTITY_ONLY.
    """

    name: str

    quantity_management_mode: Literal["SEAT_BASED", "QUANTITY_ONLY"]
    """Determines how the subscription's quantity is controlled.

    Defaults to QUANTITY_ONLY. **QUANTITY_ONLY**: The subscription quantity is
    specified directly on the subscription. `initial_quantity` must be provided with
    this option. Compatible with recurring commits/credits that use POOLED
    allocation. **SEAT_BASED**: Use when you want to pass specific seat identifiers
    (e.g. add user_123) to increment and decrement a subscription quantity, rather
    than directly providing the quantity. You must use a **SEAT_BASED** subscription
    to use a linked recurring credit with an allocation per seat. `seat_config` must
    be provided with this option.
    """

    seat_config: SubscriptionSeatConfig

    starting_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Inclusive start time for the subscription.

    If not provided, defaults to contract start date
    """

    temporary_id: str
    """
    A temporary ID used to reference the subscription in recurring commit/credit
    subscription configs created within the same payload.
    """


class TransitionFutureInvoiceBehavior(TypedDict, total=False):
    trueup: Optional[Literal["REMOVE", "AS_IS"]]
    """Controls whether future trueup invoices are billed or removed.

    Default behavior is AS_IS if not specified.
    """


class Transition(TypedDict, total=False):
    from_contract_id: Required[str]

    type: Required[Literal["SUPERSEDE", "RENEWAL"]]
    """This field's available values may vary based on your client's configuration."""

    future_invoice_behavior: TransitionFutureInvoiceBehavior


class UsageStatementSchedule(TypedDict, total=False):
    frequency: Required[Literal["MONTHLY", "QUARTERLY", "ANNUAL", "WEEKLY"]]

    billing_anchor_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Required when using CUSTOM_DATE.

    This option lets you set a historical billing anchor date, aligning future
    billing cycles with a chosen cadence. For example, if a contract starts on
    2024-09-15 and you set the anchor date to 2024-09-10 with a MONTHLY frequency,
    the first usage statement will cover 09-15 to 10-10. Subsequent statements will
    follow the 10th of each month.
    """

    day: Literal["FIRST_OF_MONTH", "CONTRACT_START", "CUSTOM_DATE"]
    """If not provided, defaults to the first day of the month."""

    invoice_generation_starting_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """The date Metronome should start generating usage invoices.

    If unspecified, contract start date will be used. This is useful to set if you
    want to import historical invoices via our 'Create Historical Invoices' API
    rather than having Metronome automatically generate them.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["UsageListWithGroupsResponse"]


class UsageListWithGroupsResponse(BaseModel):
    ending_before: datetime

    group_key: Optional[str] = None

    group_value: Optional[str] = None

    starting_on: datetime

    value: Optional[float] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ...types.v1 import invoice_void_params, invoice_regenerate_params
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.v1.invoice_void_response import InvoiceVoidResponse
from ...types.v1.invoice_regenerate_response import InvoiceRegenerateResponse

__all__ = ["InvoicesResource", "AsyncInvoicesResource"]


class InvoicesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> InvoicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return InvoicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InvoicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return InvoicesResourceWithStreamingResponse(self)

    def regenerate(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InvoiceRegenerateResponse:
        """
        This endpoint regenerates a voided invoice and recalculates the invoice based on
        up-to-date rates, available balances, and other fees regardless of the billing
        period.

        ### Use this endpoint to:

        Recalculate an invoice with updated rate terms, available balance, and fees to
        correct billing disputes or discrepancies

        ### Key response fields:

        The regenerated invoice id, which is distinct from the previously voided
        invoice.

        ### Usage guidelines:

        If an invoice is attached to a contract with a billing provider on it, the
        regenerated invoice will be distributed based on the configuration.

        Args:
          id: The invoice id to regenerate

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/invoices/regenerate",
            body=maybe_transform({"id": id}, invoice_regenerate_params.InvoiceRegenerateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InvoiceRegenerateResponse,
        )

    def void(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InvoiceVoidResponse:
        """
        Permanently cancels an invoice by setting its status to voided, preventing
        collection and removing it from customer billing. Use this to correct billing
        errors, cancel incorrect charges, or handle disputed invoices that should not be
        collected. Returns the voided invoice ID with the status change applied
        immediately to stop any payment processing.

        Args:
          id: The invoice id to void

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/invoices/void",
            body=maybe_transform({"id": id}, invoice_void_params.InvoiceVoidParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InvoiceVoidResponse,
        )


class AsyncInvoicesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncInvoicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInvoicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInvoicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncInvoicesResourceWithStreamingResponse(self)

    async def regenerate(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InvoiceRegenerateResponse:
        """
        This endpoint regenerates a voided invoice and recalculates the invoice based on
        up-to-date rates, available balances, and other fees regardless of the billing
        period.

        ### Use this endpoint to:

        Recalculate an invoice with updated rate terms, available balance, and fees to
        correct billing disputes or discrepancies

        ### Key response fields:

        The regenerated invoice id, which is distinct from the previously voided
        invoice.

        ### Usage guidelines:

        If an invoice is attached to a contract with a billing provider on it, the
        regenerated invoice will be distributed based on the configuration.

        Args:
          id: The invoice id to regenerate

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/invoices/regenerate",
            body=await async_maybe_transform({"id": id}, invoice_regenerate_params.InvoiceRegenerateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InvoiceRegenerateResponse,
        )

    async def void(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InvoiceVoidResponse:
        """
        Permanently cancels an invoice by setting its status to voided, preventing
        collection and removing it from customer billing. Use this to correct billing
        errors, cancel incorrect charges, or handle disputed invoices that should not be
        collected. Returns the voided invoice ID with the status change applied
        immediately to stop any payment processing.

        Args:
          id: The invoice id to void

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/invoices/void",
            body=await async_maybe_transform({"id": id}, invoice_void_params.InvoiceVoidParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InvoiceVoidResponse,
        )


class InvoicesResourceWithRawResponse:
    def __init__(self, invoices: InvoicesResource) -> None:
        self._invoices = invoices

        self.regenerate = to_raw_response_wrapper(
            invoices.regenerate,
        )
        self.void = to_raw_response_wrapper(
            invoices.void,
        )


class AsyncInvoicesResourceWithRawResponse:
    def __init__(self, invoices: AsyncInvoicesResource) -> None:
        self._invoices = invoices

        self.regenerate = async_to_raw_response_wrapper(
            invoices.regenerate,
        )
        self.void = async_to_raw_response_wrapper(
            invoices.void,
        )


class InvoicesResourceWithStreamingResponse:
    def __init__(self, invoices: InvoicesResource) -> None:
        self._invoices = invoices

        self.regenerate = to_streamed_response_wrapper(
            invoices.regenerate,
        )
        self.void = to_streamed_response_wrapper(
            invoices.void,
        )


class AsyncInvoicesResourceWithStreamingResponse:
    def __init__(self, invoices: AsyncInvoicesResource) -> None:
        self._invoices = invoices

        self.regenerate = async_to_streamed_response_wrapper(
            invoices.regenerate,
        )
        self.void = async_to_streamed_response_wrapper(
            invoices.void,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["CreditGrantListParams"]


class CreditGrantListParams(TypedDict, total=False):
    limit: int
    """Max number of results that should be returned"""

    next_page: str
    """Cursor that indicates where the next page of results should start."""

    credit_grant_ids: SequenceNotStr[str]
    """An array of credit grant IDs.

    If this is specified, neither credit_type_ids nor customer_ids may be specified.
    """

    credit_type_ids: SequenceNotStr[str]
    """An array of credit type IDs.

    This must not be specified if credit_grant_ids is specified.
    """

    customer_ids: SequenceNotStr[str]
    """An array of Metronome customer IDs.

    This must not be specified if credit_grant_ids is specified.
    """

    effective_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Only return credit grants that are effective before this timestamp (exclusive)."""

    not_expiring_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Only return credit grants that expire at or after this timestamp."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ....._utils import PropertyInfo

__all__ = ["NamedScheduleRetrieveParams"]


class NamedScheduleRetrieveParams(TypedDict, total=False):
    contract_id: Required[str]
    """ID of the contract whose named schedule is to be retrieved"""

    customer_id: Required[str]
    """ID of the customer whose named schedule is to be retrieved"""

    schedule_name: Required[str]
    """The identifier for the schedule to be retrieved"""

    covering_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """
    If provided, at most one schedule segment will be returned (the one that covers
    this date). If not provided, all segments will be returned.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from .commit import Commit
from .credit import Credit
from .discount import Discount
from .override import Override
from ..._models import BaseModel
from .pro_service import ProService
from .subscription import Subscription
from .scheduled_charge import ScheduledCharge
from .contract_without_amendments import ContractWithoutAmendments
from .spend_threshold_configuration import SpendThresholdConfiguration
from .prepaid_balance_threshold_configuration import PrepaidBalanceThresholdConfiguration

__all__ = ["Contract", "Amendment", "AmendmentResellerRoyalty", "CustomerBillingProviderConfiguration"]


class AmendmentResellerRoyalty(BaseModel):
    reseller_type: Literal["AWS", "AWS_PRO_SERVICE", "GCP", "GCP_PRO_SERVICE"]

    aws_account_number: Optional[str] = None

    aws_offer_id: Optional[str] = None

    aws_payer_reference_id: Optional[str] = None

    ending_before: Optional[datetime] = None

    fraction: Optional[float] = None

    gcp_account_id: Optional[str] = None

    gcp_offer_id: Optional[str] = None

    netsuite_reseller_id: Optional[str] = None

    reseller_contract_value: Optional[float] = None

    starting_at: Optional[datetime] = None


class Amendment(BaseModel):
    id: str

    commits: List[Commit]

    created_at: datetime

    created_by: str

    overrides: List[Override]

    scheduled_charges: List[ScheduledCharge]

    starting_at: datetime

    credits: Optional[List[Credit]] = None

    discounts: Optional[List[Discount]] = None
    """This field's availability is dependent on your client's configuration."""

    netsuite_sales_order_id: Optional[str] = None
    """This field's availability is dependent on your client's configuration."""

    professional_services: Optional[List[ProService]] = None
    """This field's availability is dependent on your client's configuration."""

    reseller_royalties: Optional[List[AmendmentResellerRoyalty]] = None
    """This field's availability is dependent on your client's configuration."""

    salesforce_opportunity_id: Optional[str] = None
    """This field's availability is dependent on your client's configuration."""


class CustomerBillingProviderConfiguration(BaseModel):
    """The billing provider configuration associated with a contract."""

    archived_at: Optional[datetime] = None

    billing_provider: Literal[
        "aws_marketplace",
        "stripe",
        "netsuite",
        "custom",
        "azure_marketplace",
        "quickbooks_online",
        "workday",
        "gcp_marketplace",
        "metronome",
    ]

    delivery_method: Literal["direct_to_billing_provider", "aws_sqs", "tackle", "aws_sns"]

    id: Optional[str] = None

    configuration: Optional[Dict[str, object]] = None
    """Configuration for the billing provider.

    The structure of this object is specific to the billing provider.
    """


class Contract(BaseModel):
    id: str

    amendments: List[Amendment]

    current: ContractWithoutAmendments

    customer_id: str

    initial: ContractWithoutAmendments

    archived_at: Optional[datetime] = None
    """RFC 3339 timestamp indicating when the contract was archived.

    If not returned, the contract is not archived.
    """

    custom_fields: Optional[Dict[str, str]] = None
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    customer_billing_provider_configuration: Optional[CustomerBillingProviderConfiguration] = None
    """The billing provider configuration associated with a contract."""

    prepaid_balance_threshold_configuration: Optional[PrepaidBalanceThresholdConfiguration] = None

    priority: Optional[float] = None
    """Priority of the contract."""

    scheduled_charges_on_usage_invoices: Optional[Literal["ALL"]] = None
    """
    Determines which scheduled and commit charges to consolidate onto the Contract's
    usage invoice. The charge's `timestamp` must match the usage invoice's
    `ending_before` date for consolidation to occur. This field cannot be modified
    after a Contract has been created. If this field is omitted, charges will appear
    on a separate invoice from usage charges.
    """

    spend_threshold_configuration: Optional[SpendThresholdConfiguration] = None

    subscriptions: Optional[List[Subscription]] = None
    """List of subscriptions on the contract."""

    uniqueness_key: Optional[str] = None
    """Prevents the creation of duplicates.

    If a request to create a record is made with a previously used uniqueness key, a
    new record will not be created and the request will fail with a 409 error.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["NamedScheduleUpdateParams"]


class NamedScheduleUpdateParams(TypedDict, total=False):
    customer_id: Required[str]
    """ID of the customer whose named schedule is to be updated"""

    schedule_name: Required[str]
    """The identifier for the schedule to be updated"""

    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    value: Required[object]
    """The value to set for the named schedule.

    The structure of this object is specific to the named schedule.
    """

    ending_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ...types.v1 import usage_list_params, usage_ingest_params, usage_search_params, usage_list_with_groups_params
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursorPage, AsyncCursorPage, SyncCursorPageWithoutLimit, AsyncCursorPageWithoutLimit
from ..._base_client import AsyncPaginator, make_request_options
from ...types.v1.usage_list_response import UsageListResponse
from ...types.v1.usage_search_response import UsageSearchResponse
from ...types.v1.usage_list_with_groups_response import UsageListWithGroupsResponse

__all__ = ["UsageResource", "AsyncUsageResource"]


class UsageResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return UsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return UsageResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        ending_before: Union[str, datetime],
        starting_on: Union[str, datetime],
        window_size: Literal["HOUR", "DAY", "NONE"],
        next_page: str | Omit = omit,
        billable_metrics: Iterable[usage_list_params.BillableMetric] | Omit = omit,
        customer_ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPageWithoutLimit[UsageListResponse]:
        """
        Retrieve aggregated usage data across multiple customers and billable metrics in
        a single query. This batch endpoint enables you to fetch usage patterns at
        scale, broken down by time windows, making it ideal for building analytics
        dashboards, generating reports, and monitoring platform-wide usage trends.

        ### Use this endpoint to:

        - Generate platform-wide usage reports for internal teams
        - Monitor aggregate usage trends across your entire customer base
        - Create comparative usage analyses between customers or time periods
        - Support capacity planning with historical usage patterns

        ### Key response fields:

        An array of `UsageBatchAggregate` objects containing:

        - `customer_id`: The customer this usage belongs to
        - `billable_metric_id` and `billable_metric_name`: What was measured
        - `start_timestamp` and `end_timestamp`: Time window for this data point
        - `value`: Aggregated usage amount for the period
        - `groups` (optional): Usage broken down by group keys with values
        - `next_page`: Pagination cursor for large result sets

        ### Usage guidelines:

        - Time windows: Set `window_size` to `hour`, `day`, or `none` (entire period)
        - Required parameters: Must specify `starting_on`, `ending_before`, and
          `window_size`
        - Filtering options:
          - `customer_ids`: Limit to specific customers (omit for all customers)
          - `billable_metrics`: Limit to specific metrics (omit for all metrics)
        - Pagination: Use `next_page` cursor to retrieve large datasets
        - Null values: Group values may be null when no usage matches that group

        Args:
          window_size: A window_size of "day" or "hour" will return the usage for the specified period
              segmented into daily or hourly aggregates. A window_size of "none" will return a
              single usage aggregate for the entirety of the specified period.

          next_page: Cursor that indicates where the next page of results should start.

          billable_metrics: A list of billable metrics to fetch usage for. If absent, all billable metrics
              will be returned.

          customer_ids: A list of Metronome customer IDs to fetch usage for. If absent, usage for all
              customers will be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/usage",
            page=SyncCursorPageWithoutLimit[UsageListResponse],
            body=maybe_transform(
                {
                    "ending_before": ending_before,
                    "starting_on": starting_on,
                    "window_size": window_size,
                    "billable_metrics": billable_metrics,
                    "customer_ids": customer_ids,
                },
                usage_list_params.UsageListParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"next_page": next_page}, usage_list_params.UsageListParams),
            ),
            model=UsageListResponse,
            method="post",
        )

    def ingest(
        self,
        *,
        usage: Iterable[usage_ingest_params.Usage] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        The ingest endpoint is the primary method for sending usage events to Metronome,
        serving as the foundation for all billing calculations in your usage-based
        pricing model. This high-throughput endpoint is designed for real-time streaming
        ingestion, supports backdating 34 days, and is built to handle mission-critical
        usage data with enterprise-grade reliability. Metronome supports 100,000 events
        per second without requiring pre-aggregation or rollups and can scale up from
        there. See the [Send usage events](/guides/events/send-usage-events) guide to
        learn more about usage events.

        ### Use this endpoint to:

        Create a customer usage pipeline into Metronome that drives billable metrics,
        credit drawdown, and invoicing. Track customer behavior, resource consumption,
        and feature usage

        ### What happens when you send events:

        - Events are validated and processed in real-time
        - Events are matched to customers using customer IDs or customer ingest aliases
        - Events are matched to billable metrics and are immediately available for usage
          and spend calculations

        ### Usage guidelines:

        - Historical events can be backdated up to 34 days and will immediately impact
          live customer spend
        - Duplicate events are automatically detected and ignored (34-day deduplication
          window)

        #### Event structure:

        Usage events are simple JSON objects designed for flexibility and ease of
        integration:

        ```json
        {
          "transaction_id": "2021-01-01T00:00:00Z_cluster42",
          "customer_id": "team@example.com",
          "event_type": "api_request",
          "timestamp": "2021-01-01T00:00:00Z",
          "properties": {
            "endpoint": "/v1/users",
            "method": "POST",
            "response_time_ms": 45,
            "region": "us-west-2"
          }
        }
        ```

        Learn more about
        [usage event structure definitions](/guides/events/design-usage-events).

        #### Transaction ID

        The transaction_id serves as your idempotency key, ensuring events are processed
        exactly once. Metronome maintains a 34-day deduplication window - significantly
        longer than typical 12-hour windows - enabling robust backfill scenarios without
        duplicate billing.

        - Best Practices:
          - Use UUIDs for one-time events: uuid4()
          - For heartbeat events, use deterministic IDs
          - Include enough context to avoid collisions across different event sources

        #### Customer ID

        Identifies which customer should be billed for this usage. Supports two
        identification methods:

        - Metronome Customer ID: The UUID returned when creating a customer
        - Ingest Alias: Your system's identifier (email, account number, etc.)

        Ingest aliases enable seamless integration without requiring ID mapping, and
        customers can have multiple aliases for flexibility.

        #### Event Type:

        Categorizes the event type for billable metric matching. Choose descriptive
        names that aligns with the product surface area.

        #### Properties:

        Flexible metadata also used to match billable metrics or to be used to serve as
        group keys to create multiple pricing dimensions or breakdown costs by novel
        properties for end customers or internal finance teams measuring underlying
        COGs.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            "/v1/ingest",
            body=maybe_transform(usage, Iterable[usage_ingest_params.Usage]),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def list_with_groups(
        self,
        *,
        billable_metric_id: str,
        customer_id: str,
        window_size: Literal["HOUR", "DAY", "NONE"],
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        current_period: bool | Omit = omit,
        ending_before: Union[str, datetime] | Omit = omit,
        group_by: usage_list_with_groups_params.GroupBy | Omit = omit,
        starting_on: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[UsageListWithGroupsResponse]:
        """
        Retrieve granular usage data for a specific customer and billable metric, with
        the ability to break down usage by custom grouping dimensions. This endpoint
        enables deep usage analytics by segmenting data across attributes like region,
        user, model type, or any custom dimension defined in your billable metrics.

        ### Use this endpoint to:

        - Analyze usage patterns broken down by specific attributes (region, user,
          department, etc.)
        - Build detailed usage dashboards with dimensional filtering
        - Identify high-usage segments for optimization opportunities

        ### Key response fields:

        An array of `PagedUsageAggregate` objects containing:

        - `starting_on` and `ending_before`: Time window boundaries
        - `group_key`: The dimension being grouped by (e.g., "region")
        - `group_value`: The specific value for this group (e.g., "US-East")
        - `value`: Aggregated usage for this group and time window
        - `next_page`: Pagination cursor for large datasets

        ### Usage guidelines:

        - Required parameters: Must specify `customer_id`, `billable_metric_id`, and
          `window_size`
        - Time windows: Set `window_size` to hour, day, or none for different
          granularities
        - Group filtering: Use `group_by` to specify:
          - key: The dimension to group by (must be set on the billable metric as a
            group key)
          - values: Optional array to filter to specific values only
        - Pagination: Use limit and `next_page` for large result sets
        - Null handling: `group_value` may be null for unmatched data

        Args:
          window_size: A window_size of "day" or "hour" will return the usage for the specified period
              segmented into daily or hourly aggregates. A window_size of "none" will return a
              single usage aggregate for the entirety of the specified period.

          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          current_period: If true, will return the usage for the current billing period. Will return an
              error if the customer is currently uncontracted or starting_on and ending_before
              are specified when this is true.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/usage/groups",
            page=SyncCursorPage[UsageListWithGroupsResponse],
            body=maybe_transform(
                {
                    "billable_metric_id": billable_metric_id,
                    "customer_id": customer_id,
                    "window_size": window_size,
                    "current_period": current_period,
                    "ending_before": ending_before,
                    "group_by": group_by,
                    "starting_on": starting_on,
                },
                usage_list_with_groups_params.UsageListWithGroupsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    usage_list_with_groups_params.UsageListWithGroupsParams,
                ),
            ),
            model=UsageListWithGroupsResponse,
            method="post",
        )

    def search(
        self,
        *,
        transaction_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageSearchResponse:
        """
        This endpoint retrieves events by transaction ID for events that occurred within
        the last 34 days. It is specifically designed for sampling-based testing
        workflows to detect revenue leakage. The Event Search API provides a critical
        observability tool that validates the integrity of your usage pipeline by
        allowing you to sample raw events and verify their matching against active
        billable metrics.

        Why event observability matters for revenue leakage: Silent revenue loss occurs
        when events are dropped, delayed, or fail to match billable metrics due to:

        - Upstream system failures
        - Event format changes
        - Misconfigured billable metrics

        ### Use this endpoint to:

        - Sample raw events and validate they match the expected billable metrics
        - Build custom leakage detection alerts to prevent silent revenue loss
        - Verify event processing accuracy during system changes or metric updates
        - Debug event matching issues in real-time

        ### Key response fields:

        - Complete event details including transaction ID, customer ID, and properties
        - Matched Metronome customer (if any)
        - Matched billable metric information (if any)
        - Processing status and duplicate detection flags

        ### Usage guidelines:

        ⚠️ Important: This endpoint is heavily rate limited and designed for sampling
        workflows only. Do not use this endpoint to check every event in your system.
        Instead, implement a sampling strategy to randomly validate a subset of events
        for observability purposes.

        Args:
          transaction_ids: The transaction IDs of the events to retrieve

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/events/search",
            body=maybe_transform({"transaction_ids": transaction_ids}, usage_search_params.UsageSearchParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UsageSearchResponse,
        )


class AsyncUsageResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncUsageResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        ending_before: Union[str, datetime],
        starting_on: Union[str, datetime],
        window_size: Literal["HOUR", "DAY", "NONE"],
        next_page: str | Omit = omit,
        billable_metrics: Iterable[usage_list_params.BillableMetric] | Omit = omit,
        customer_ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[UsageListResponse, AsyncCursorPageWithoutLimit[UsageListResponse]]:
        """
        Retrieve aggregated usage data across multiple customers and billable metrics in
        a single query. This batch endpoint enables you to fetch usage patterns at
        scale, broken down by time windows, making it ideal for building analytics
        dashboards, generating reports, and monitoring platform-wide usage trends.

        ### Use this endpoint to:

        - Generate platform-wide usage reports for internal teams
        - Monitor aggregate usage trends across your entire customer base
        - Create comparative usage analyses between customers or time periods
        - Support capacity planning with historical usage patterns

        ### Key response fields:

        An array of `UsageBatchAggregate` objects containing:

        - `customer_id`: The customer this usage belongs to
        - `billable_metric_id` and `billable_metric_name`: What was measured
        - `start_timestamp` and `end_timestamp`: Time window for this data point
        - `value`: Aggregated usage amount for the period
        - `groups` (optional): Usage broken down by group keys with values
        - `next_page`: Pagination cursor for large result sets

        ### Usage guidelines:

        - Time windows: Set `window_size` to `hour`, `day`, or `none` (entire period)
        - Required parameters: Must specify `starting_on`, `ending_before`, and
          `window_size`
        - Filtering options:
          - `customer_ids`: Limit to specific customers (omit for all customers)
          - `billable_metrics`: Limit to specific metrics (omit for all metrics)
        - Pagination: Use `next_page` cursor to retrieve large datasets
        - Null values: Group values may be null when no usage matches that group

        Args:
          window_size: A window_size of "day" or "hour" will return the usage for the specified period
              segmented into daily or hourly aggregates. A window_size of "none" will return a
              single usage aggregate for the entirety of the specified period.

          next_page: Cursor that indicates where the next page of results should start.

          billable_metrics: A list of billable metrics to fetch usage for. If absent, all billable metrics
              will be returned.

          customer_ids: A list of Metronome customer IDs to fetch usage for. If absent, usage for all
              customers will be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/usage",
            page=AsyncCursorPageWithoutLimit[UsageListResponse],
            body=maybe_transform(
                {
                    "ending_before": ending_before,
                    "starting_on": starting_on,
                    "window_size": window_size,
                    "billable_metrics": billable_metrics,
                    "customer_ids": customer_ids,
                },
                usage_list_params.UsageListParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"next_page": next_page}, usage_list_params.UsageListParams),
            ),
            model=UsageListResponse,
            method="post",
        )

    async def ingest(
        self,
        *,
        usage: Iterable[usage_ingest_params.Usage] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        The ingest endpoint is the primary method for sending usage events to Metronome,
        serving as the foundation for all billing calculations in your usage-based
        pricing model. This high-throughput endpoint is designed for real-time streaming
        ingestion, supports backdating 34 days, and is built to handle mission-critical
        usage data with enterprise-grade reliability. Metronome supports 100,000 events
        per second without requiring pre-aggregation or rollups and can scale up from
        there. See the [Send usage events](/guides/events/send-usage-events) guide to
        learn more about usage events.

        ### Use this endpoint to:

        Create a customer usage pipeline into Metronome that drives billable metrics,
        credit drawdown, and invoicing. Track customer behavior, resource consumption,
        and feature usage

        ### What happens when you send events:

        - Events are validated and processed in real-time
        - Events are matched to customers using customer IDs or customer ingest aliases
        - Events are matched to billable metrics and are immediately available for usage
          and spend calculations

        ### Usage guidelines:

        - Historical events can be backdated up to 34 days and will immediately impact
          live customer spend
        - Duplicate events are automatically detected and ignored (34-day deduplication
          window)

        #### Event structure:

        Usage events are simple JSON objects designed for flexibility and ease of
        integration:

        ```json
        {
          "transaction_id": "2021-01-01T00:00:00Z_cluster42",
          "customer_id": "team@example.com",
          "event_type": "api_request",
          "timestamp": "2021-01-01T00:00:00Z",
          "properties": {
            "endpoint": "/v1/users",
            "method": "POST",
            "response_time_ms": 45,
            "region": "us-west-2"
          }
        }
        ```

        Learn more about
        [usage event structure definitions](/guides/events/design-usage-events).

        #### Transaction ID

        The transaction_id serves as your idempotency key, ensuring events are processed
        exactly once. Metronome maintains a 34-day deduplication window - significantly
        longer than typical 12-hour windows - enabling robust backfill scenarios without
        duplicate billing.

        - Best Practices:
          - Use UUIDs for one-time events: uuid4()
          - For heartbeat events, use deterministic IDs
          - Include enough context to avoid collisions across different event sources

        #### Customer ID

        Identifies which customer should be billed for this usage. Supports two
        identification methods:

        - Metronome Customer ID: The UUID returned when creating a customer
        - Ingest Alias: Your system's identifier (email, account number, etc.)

        Ingest aliases enable seamless integration without requiring ID mapping, and
        customers can have multiple aliases for flexibility.

        #### Event Type:

        Categorizes the event type for billable metric matching. Choose descriptive
        names that aligns with the product surface area.

        #### Properties:

        Flexible metadata also used to match billable metrics or to be used to serve as
        group keys to create multiple pricing dimensions or breakdown costs by novel
        properties for end customers or internal finance teams measuring underlying
        COGs.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            "/v1/ingest",
            body=await async_maybe_transform(usage, Iterable[usage_ingest_params.Usage]),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def list_with_groups(
        self,
        *,
        billable_metric_id: str,
        customer_id: str,
        window_size: Literal["HOUR", "DAY", "NONE"],
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        current_period: bool | Omit = omit,
        ending_before: Union[str, datetime] | Omit = omit,
        group_by: usage_list_with_groups_params.GroupBy | Omit = omit,
        starting_on: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[UsageListWithGroupsResponse, AsyncCursorPage[UsageListWithGroupsResponse]]:
        """
        Retrieve granular usage data for a specific customer and billable metric, with
        the ability to break down usage by custom grouping dimensions. This endpoint
        enables deep usage analytics by segmenting data across attributes like region,
        user, model type, or any custom dimension defined in your billable metrics.

        ### Use this endpoint to:

        - Analyze usage patterns broken down by specific attributes (region, user,
          department, etc.)
        - Build detailed usage dashboards with dimensional filtering
        - Identify high-usage segments for optimization opportunities

        ### Key response fields:

        An array of `PagedUsageAggregate` objects containing:

        - `starting_on` and `ending_before`: Time window boundaries
        - `group_key`: The dimension being grouped by (e.g., "region")
        - `group_value`: The specific value for this group (e.g., "US-East")
        - `value`: Aggregated usage for this group and time window
        - `next_page`: Pagination cursor for large datasets

        ### Usage guidelines:

        - Required parameters: Must specify `customer_id`, `billable_metric_id`, and
          `window_size`
        - Time windows: Set `window_size` to hour, day, or none for different
          granularities
        - Group filtering: Use `group_by` to specify:
          - key: The dimension to group by (must be set on the billable metric as a
            group key)
          - values: Optional array to filter to specific values only
        - Pagination: Use limit and `next_page` for large result sets
        - Null handling: `group_value` may be null for unmatched data

        Args:
          window_size: A window_size of "day" or "hour" will return the usage for the specified period
              segmented into daily or hourly aggregates. A window_size of "none" will return a
              single usage aggregate for the entirety of the specified period.

          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          current_period: If true, will return the usage for the current billing period. Will return an
              error if the customer is currently uncontracted or starting_on and ending_before
              are specified when this is true.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/usage/groups",
            page=AsyncCursorPage[UsageListWithGroupsResponse],
            body=maybe_transform(
                {
                    "billable_metric_id": billable_metric_id,
                    "customer_id": customer_id,
                    "window_size": window_size,
                    "current_period": current_period,
                    "ending_before": ending_before,
                    "group_by": group_by,
                    "starting_on": starting_on,
                },
                usage_list_with_groups_params.UsageListWithGroupsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    usage_list_with_groups_params.UsageListWithGroupsParams,
                ),
            ),
            model=UsageListWithGroupsResponse,
            method="post",
        )

    async def search(
        self,
        *,
        transaction_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageSearchResponse:
        """
        This endpoint retrieves events by transaction ID for events that occurred within
        the last 34 days. It is specifically designed for sampling-based testing
        workflows to detect revenue leakage. The Event Search API provides a critical
        observability tool that validates the integrity of your usage pipeline by
        allowing you to sample raw events and verify their matching against active
        billable metrics.

        Why event observability matters for revenue leakage: Silent revenue loss occurs
        when events are dropped, delayed, or fail to match billable metrics due to:

        - Upstream system failures
        - Event format changes
        - Misconfigured billable metrics

        ### Use this endpoint to:

        - Sample raw events and validate they match the expected billable metrics
        - Build custom leakage detection alerts to prevent silent revenue loss
        - Verify event processing accuracy during system changes or metric updates
        - Debug event matching issues in real-time

        ### Key response fields:

        - Complete event details including transaction ID, customer ID, and properties
        - Matched Metronome customer (if any)
        - Matched billable metric information (if any)
        - Processing status and duplicate detection flags

        ### Usage guidelines:

        ⚠️ Important: This endpoint is heavily rate limited and designed for sampling
        workflows only. Do not use this endpoint to check every event in your system.
        Instead, implement a sampling strategy to randomly validate a subset of events
        for observability purposes.

        Args:
          transaction_ids: The transaction IDs of the events to retrieve

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/events/search",
            body=await async_maybe_transform(
                {"transaction_ids": transaction_ids}, usage_search_params.UsageSearchParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UsageSearchResponse,
        )


class UsageResourceWithRawResponse:
    def __init__(self, usage: UsageResource) -> None:
        self._usage = usage

        self.list = to_raw_response_wrapper(
            usage.list,
        )
        self.ingest = to_raw_response_wrapper(
            usage.ingest,
        )
        self.list_with_groups = to_raw_response_wrapper(
            usage.list_with_groups,
        )
        self.search = to_raw_response_wrapper(
            usage.search,
        )


class AsyncUsageResourceWithRawResponse:
    def __init__(self, usage: AsyncUsageResource) -> None:
        self._usage = usage

        self.list = async_to_raw_response_wrapper(
            usage.list,
        )
        self.ingest = async_to_raw_response_wrapper(
            usage.ingest,
        )
        self.list_with_groups = async_to_raw_response_wrapper(
            usage.list_with_groups,
        )
        self.search = async_to_raw_response_wrapper(
            usage.search,
        )


class UsageResourceWithStreamingResponse:
    def __init__(self, usage: UsageResource) -> None:
        self._usage = usage

        self.list = to_streamed_response_wrapper(
            usage.list,
        )
        self.ingest = to_streamed_response_wrapper(
            usage.ingest,
        )
        self.list_with_groups = to_streamed_response_wrapper(
            usage.list_with_groups,
        )
        self.search = to_streamed_response_wrapper(
            usage.search,
        )


class AsyncUsageResourceWithStreamingResponse:
    def __init__(self, usage: AsyncUsageResource) -> None:
        self._usage = usage

        self.list = async_to_streamed_response_wrapper(
            usage.list,
        )
        self.ingest = async_to_streamed_response_wrapper(
            usage.ingest,
        )
        self.list_with_groups = async_to_streamed_response_wrapper(
            usage.list_with_groups,
        )
        self.search = async_to_streamed_response_wrapper(
            usage.search,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["AlertCreateParams", "CustomFieldFilter", "GroupValue", "SeatFilter"]


class AlertCreateParams(TypedDict, total=False):
    alert_type: Required[
        Literal[
            "low_credit_balance_reached",
            "spend_threshold_reached",
            "monthly_invoice_total_spend_threshold_reached",
            "low_remaining_days_in_plan_reached",
            "low_remaining_credit_percentage_reached",
            "usage_threshold_reached",
            "low_remaining_days_for_commit_segment_reached",
            "low_remaining_commit_balance_reached",
            "low_remaining_commit_percentage_reached",
            "low_remaining_days_for_contract_credit_segment_reached",
            "low_remaining_contract_credit_balance_reached",
            "low_remaining_contract_credit_percentage_reached",
            "low_remaining_contract_credit_and_commit_balance_reached",
            "invoice_total_reached",
            "low_remaining_seat_balance_reached",
        ]
    ]
    """Type of the threshold notification"""

    name: Required[str]
    """Name of the threshold notification"""

    threshold: Required[float]
    """Threshold value of the notification policy.

    Depending upon the notification type, this number may represent a financial
    amount, the days remaining, or a percentage reached.
    """

    billable_metric_id: str
    """
    For threshold notifications of type `usage_threshold_reached`, specifies which
    billable metric to track the usage for.
    """

    credit_grant_type_filters: SequenceNotStr[str]
    """
    An array of strings, representing a way to filter the credit grant this
    threshold notification applies to, by looking at the credit_grant_type field on
    the credit grant. This field is only defined for CreditPercentage and
    CreditBalance notifications
    """

    credit_type_id: str
    """ID of the credit's currency, defaults to USD.

    If the specific notification type requires a pricing unit/currency, find the ID
    in the [Metronome app](https://app.metronome.com/offering/pricing-units).
    """

    custom_field_filters: Iterable[CustomFieldFilter]
    """
    A list of custom field filters for threshold notification types that support
    advanced filtering. Only present for contract invoices.
    """

    customer_id: str
    """If provided, will create this threshold notification for this specific customer.

    To create a notification for all customers, do not specify a `customer_id`.
    """

    evaluate_on_create: bool
    """
    If true, the threshold notification will evaluate immediately on customers that
    already meet the notification threshold. If false, it will only evaluate on
    future customers that trigger the threshold. Defaults to true.
    """

    group_values: Iterable[GroupValue]
    """Only present for `spend_threshold_reached` notifications.

    Scope notification to a specific group key on individual line items.
    """

    invoice_types_filter: SequenceNotStr[str]
    """Only supported for invoice_total_reached threshold notifications.

    A list of invoice types to evaluate.
    """

    plan_id: str
    """If provided, will create this threshold notification for this specific plan.

    To create a notification for all customers, do not specify a `plan_id`.
    """

    seat_filter: SeatFilter
    """Required for `low_remaining_seat_balance_reached` notifications.

    The alert is scoped to this seat group key-value pair.
    """

    uniqueness_key: str
    """Prevents the creation of duplicates.

    If a request to create a record is made with a previously used uniqueness key, a
    new record will not be created and the request will fail with a 409 error.
    """


class CustomFieldFilter(TypedDict, total=False):
    entity: Required[Literal["Contract", "Commit", "ContractCredit"]]

    key: Required[str]

    value: Required[str]


class GroupValue(TypedDict, total=False):
    key: Required[str]

    value: str


class SeatFilter(TypedDict, total=False):
    """Required for `low_remaining_seat_balance_reached` notifications.

    The alert is scoped to this seat group key-value pair.
    """

    seat_group_key: Required[str]
    """The seat group key (e.g., "seat_id", "user_id")"""

    seat_group_value: str
    """Optional seat identifier the alert is scoped to."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr

__all__ = [
    "CommitHierarchyConfiguration",
    "ChildAccess",
    "ChildAccessCommitHierarchyChildAccessAll",
    "ChildAccessCommitHierarchyChildAccessNone",
    "ChildAccessCommitHierarchyChildAccessContractIDs",
]


class ChildAccessCommitHierarchyChildAccessAll(TypedDict, total=False):
    type: Required[Literal["ALL"]]


class ChildAccessCommitHierarchyChildAccessNone(TypedDict, total=False):
    type: Required[Literal["NONE"]]


class ChildAccessCommitHierarchyChildAccessContractIDs(TypedDict, total=False):
    contract_ids: Required[SequenceNotStr[str]]

    type: Required[Literal["CONTRACT_IDS"]]


ChildAccess: TypeAlias = Union[
    ChildAccessCommitHierarchyChildAccessAll,
    ChildAccessCommitHierarchyChildAccessNone,
    ChildAccessCommitHierarchyChildAccessContractIDs,
]


class CommitHierarchyConfiguration(TypedDict, total=False):
    child_access: Required[ChildAccess]
